<?php

/* --------------------------------------------------------------
   ContentWriter.inc.php 2019-04-16
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2019 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

/**
 * Class ContentWriter
 *
 * This class represents the content writer
 *
 * @category   System
 * @package    Content
 */
class ContentWriter implements ContentWriterInterface
{
    /**
     * CodeIgniter QueryBuilder
     *
     * @var CI_DB_query_builder
     */
    protected $queryBuilder;
    
    /**
     * Language map
     *
     * @var array
     */
    protected $languageMap;
    
    /**
     * Data
     *
     * @var array
     */
    protected $data = [];
    
    
    /**
     * ContentManagerWriter constructor
     *
     * @param CI_DB_query_builder $queryBuilder CodeIgniter QueryBuilder
     */
    public function __construct(CI_DB_query_builder $queryBuilder)
    {
        $this->queryBuilder = $queryBuilder;
    }
    
    
    /**
     * Map the languages
     */
    protected function mapLanguages()
    {
        if ($this->languageMap === null) {
            $result = $this->queryBuilder->select('languages_id, code')->from('languages')->get()->result_array();
            
            foreach ($result as $language) {
                $this->languageMap[$language['code']] = (int)$language['languages_id'];
            }
        }
    }
    
    
    /**
     * Return the highest content group ID and add one
     *
     * @return string
     */
    protected function nextId()
    {
        $result = $this->queryBuilder->select('MAX(content_group)')
            ->from('content_manager')
            ->get()
            ->result_array();
        $lastResult = array_pop($result);
        $contentGroupId = (int)array_pop($lastResult);
        $nextId = $contentGroupId + 1;
        
        return (string)$nextId;
    }
    
    
    /**
     * Store an info element content into the database
     *
     * @param InfoElementContent $infoElementContent Info element content
     *
     * @return string New content group ID
     *
     * @throws LocalizedAttributeNotFoundException On missing locale-based entry
     */
    public function storeInfoElementContent(InfoElementContent $infoElementContent)
    {
        $this->mapLanguages();
        
        $contentGroupId = $this->nextId();
        
        foreach ($this->languageMap as $languageCode => $languageCodeId) {
            $languageCodeObject = new LanguageCode(new StringType($languageCode));
            $this->data = [];
            
            $this->data['content_delete'] = '1';
            
            $this->data['content_heading'] = $infoElementContent->headings()
                ->itemByLanguageCode($languageCodeObject)
                ->content();
            $this->data['content_title'] = $infoElementContent->titles()
                ->itemByLanguageCode($languageCodeObject)
                ->content();
            $this->data['content_text'] = $infoElementContent->texts()
                ->itemByLanguageCode($languageCodeObject)
                ->content();
            $this->data['content_status'] = (string)(int)$infoElementContent->isActive();
            $this->data['content_position'] = ElementPositionMapper::getElementPositionForDatabase(new StringType($infoElementContent->position()));
            $this->data['content_group'] = $contentGroupId;
            $this->data['content_type'] = 'content';
            $this->data['languages_id'] = $this->languageMap[$languageCode];
            
            $this->queryBuilder->insert('content_manager', $this->data);
        }
        
        return $contentGroupId;
    }
    
    
    /**
     * Store an info page content into the database
     *
     * @param InfoPageContent $infoPageContent Info page content
     *
     * @return string New content group ID
     *
     * @throws LocalizedAttributeNotFoundException On missing locale-based entry
     */
    public function storeInfoPageContent(InfoPageContent $infoPageContent)
    {
        $this->mapLanguages();
        
        $contentGroupId = $this->nextId();
        
        foreach ($this->languageMap as $languageCode => $languageCodeId) {
            $languageCodeObject = new LanguageCode(new StringType($languageCode));
            $this->data = [];
            
            $this->data['content_delete'] = '1';
            $this->data['content_status'] = (string)(int)$infoPageContent->isActive();
            $this->data['content_group'] = $contentGroupId;
            $this->data['content_type'] = 'content';
            $this->data['content_heading'] = $infoPageContent->headings()
                ->itemByLanguageCode($languageCodeObject)
                ->content();
            $this->data['content_text'] = $infoPageContent->texts()
                ->itemByLanguageCode($languageCodeObject)
                ->content();
            $this->data['languages_id'] = $this->languageMap[$languageCode];
            $this->data['content_position'] = PagePositionMapper::getPagePositionForDatabase(new StringType($infoPageContent->position()));
            $this->data['contents_meta_title'] = $infoPageContent->metaTitles()
                ->itemByLanguageCode($languageCodeObject)
                ->content();
            $this->data['contents_meta_description'] = $infoPageContent->metaDescriptions()
                ->itemByLanguageCode($languageCodeObject)
                ->content();
            $this->data['content_title'] = $infoPageContent->titles()
                ->itemByLanguageCode($languageCodeObject)
                ->content();
            $this->data['contents_meta_keywords'] = $infoPageContent->metaKeywords()
                ->itemByLanguageCode($languageCodeObject)
                ->content();
            $this->data['download_file'] = $infoPageContent->downloads()
                ->itemByLanguageCode($languageCodeObject)
                ->content();
            $this->data['opengraph_image'] = $infoPageContent->opengraphImages()
                ->itemByLanguageCode($languageCodeObject)
                ->content();
            $this->data['content_name'] = $infoPageContent->names()
                ->itemByLanguageCode($languageCodeObject)
                ->content();
            $this->data['gm_url_keywords'] = $infoPageContent->urlKeywords()
                ->itemByLanguageCode($languageCodeObject)
                ->content();
            $this->data['gm_changefreq'] = $infoPageContent->sitemap()->frequencyOfChange();
            $this->data['gm_priority'] = $infoPageContent->sitemap()
                ->priorities()
                ->itemByLanguageCode($languageCodeObject)
                ->content();
            $this->data['gm_sitemap_entry'] = $infoPageContent->sitemap()->isVisible();
            
            $this->queryBuilder->insert('content_manager', $this->data);
            
            $insertId = (string)$this->queryBuilder->insert_id();
            $this->data = [];
            
            $this->data['content_id'] = $insertId;
            $this->data['content_type'] = 'content';
            $this->data['language_id'] = $this->languageMap[$languageCode];
            $this->data['rewrite_url'] = $infoPageContent->urlRewrites()
                ->itemByLanguageCode($languageCodeObject)
                ->content();
            $this->data['target_url'] = '';
            
            $this->queryBuilder->insert('url_rewrites', $this->data);
        }
        
        return $contentGroupId;
    }
    
    
    /**
     * Store a link page content into the database
     *
     * @param LinkPageContent $linkPageContent Link page content
     *
     * @return string New content group ID
     *
     * @throws LocalizedAttributeNotFoundException On missing locale-based entry
     */
    public function storeLinkPageContent(LinkPageContent $linkPageContent)
    {
        $this->mapLanguages();
        
        $contentGroupId = $this->nextId();
        
        foreach ($this->languageMap as $languageCode => $languageCodeId) {
            $languageCodeObject = new LanguageCode(new StringType($languageCode));
            $this->data = [];
            
            $this->data['content_delete'] = '1';
            $this->data['content_type'] = 'link';
            $this->data['content_status'] = (string)(int)$linkPageContent->isActive();
            $this->data['content_group'] = $contentGroupId;
            $this->data['content_type'] = 'content';
            $this->data['languages_id'] = $this->languageMap[$languageCode];
            $this->data['content_position'] = $linkPageContent->position();
            $this->data['content_title'] = $linkPageContent->titles()
                ->itemByLanguageCode($languageCodeObject)
                ->content();
            $this->data['content_name'] = $linkPageContent->names()
                ->itemByLanguageCode($languageCodeObject)
                ->content();
            $this->data['gm_link_target'] = $linkPageContent->isOpeningInNewTab() ? '_blank' : '_top';
            $this->data['gm_link'] = $linkPageContent->links()
                ->itemByLanguageCode($languageCodeObject)
                ->content();
            
            $this->queryBuilder->insert('content_manager', $this->data);
        }
        
        return $contentGroupId;
    }
}