<?php
/* --------------------------------------------------------------
   ThemeWriter.php 2019-06-06
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2019 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/


use League\Flysystem\FileExistsException;

/**
 * Class ThemeWriter
 */
class ThemeWriter implements ThemeWriterInterface
{
    const FILE_IS_SCRIPT_OR_SHEET_PATTERN = '#\.(css|js)$#i';

    const UNMINIFIED_FILE_NAME_PATTERN = '#(\.min)?\.(css|js)$#i';

    protected $filesystem;

    protected $shopSource;

    protected $sourceThemeId;

    /**
     * ThemeWriter constructor.
     * @param FilesystemAdapter $filesystem
     * @param ExistingDirectory $shopSource
     */
    public function __construct(FilesystemAdapter $filesystem, ExistingDirectory $shopSource)
    {
        $this->filesystem = $filesystem;
        $this->shopSource = $shopSource->getAbsolutePath();
    }


    /**
     * Saves the given them to the destination.
     *
     * @param ThemeInterface $theme Theme to be saved.
     * @param ThemeDirectoryRootInterface $destination Destination directory.
     *
     * @return void
     * @throws Exception
     */
    public function save(ThemeInterface $theme, ThemeDirectoryRootInterface $destination)
    {
        $this->sourceThemeId = $theme->getId();

        $destinationDirectory = $this->_getRelative($destination);
        $contents = $this->filesystem->listContents($destinationDirectory);

        // delete all files and directories in the theme folder except for the index.html
        foreach ($contents as $content) {
            $path = '/' . $content['path'];
            if ($content['type'] === 'dir') {
                $this->filesystem->deleteDir($path);
            } elseif ($path !== $destinationDirectory . '/index.html') {
                $this->filesystem->delete('/' . $content['path']);
            }
        }

        $this->filesystem->createDir($destinationDirectory, ['visibility' => 'public']);

        $childThemes = [];

        while ($theme->hasParent()) {
            $childThemes[] = $theme;
            $theme = $theme->getParentTheme();
        }
        // at this point, $theme is the main theme and $childThemes contains all child themes, reversed.

        $childThemes = array_reverse($childThemes);
        $this->_saveThemeStructure($theme->toMainTheme(), $destination, ...$childThemes);
        //  Selecting which theme.json needs to be copied
        $activeTheme = count($childThemes) > 0 ? array_pop($childThemes) : $theme;
        $this->_copyThemeJson($activeTheme, $destination);

        $this->findFilesThatHaveNoMatchingMinOrSourceFile($destination);
    }

    /**
     * @param ThemeDirectoryRootInterface $directory
     * @return mixed
     */
    protected function _getRelative(ThemeDirectoryRootInterface $directory)
    {
        return str_replace($this->shopSource, '', $directory->getPath());
    }

    /**
     * @param MainThemeInterface $mainTheme
     * @param ThemeDirectoryRootInterface $destination
     * @param ThemeInterface ...$childThemes
     *
     * @throws FileNotFoundException
     */
    protected function _saveThemeStructure(
        MainThemeInterface $mainTheme,
        ThemeDirectoryRootInterface $destination,
        ThemeInterface ...$childThemes
    ) {
        $this->_saveMainTheme($mainTheme, $destination);

        $childCount = 0;
        foreach ($childThemes as $childTheme) {
            $this->_saveTheme($childTheme, $destination);
            $childCount++;
        }
        $mainTheme = $childCount ? $childThemes[$childCount - 1] : $mainTheme;
    }

    /**
     * @param MainThemeInterface $mainTheme
     * @param ThemeDirectoryRootInterface $destination
     */
    protected function _saveMainTheme(MainThemeInterface $mainTheme, ThemeDirectoryRootInterface $destination)
    {
        $this->_copyMainThemeDirectory($mainTheme->getId(), $mainTheme->getConfig(), $destination);
        $this->_copyMainThemeDirectory($mainTheme->getId(), $mainTheme->getFonts(), $destination);
        $this->_copyMainThemeDirectory($mainTheme->getId(), $mainTheme->getHtml(), $destination);
        $this->_copyMainThemeDirectory($mainTheme->getId(), $mainTheme->getImages(), $destination);
        $this->_copyMainThemeDirectory($mainTheme->getId(), $mainTheme->getJs(), $destination);
        $this->_copyMainThemeDirectory($mainTheme->getId(), $mainTheme->getStyles(), $destination);
        $this->_copyMainThemeStyleEditDirectory($mainTheme->getId(), $mainTheme->getStyleEdit(), $destination);
    }

    /**
     * @param ThemeId $themeId
     * @param ThemeDirectoryInterface $directory
     * @param ThemeDirectoryRootInterface $destination
     */
    protected function _copyMainThemeDirectory(
        ThemeId $themeId,
        ThemeDirectoryInterface $directory,
        ThemeDirectoryRootInterface $destination
    ) {
        $childDirectories = $directory->getChildren();

        if ($childDirectories) {
            foreach ($childDirectories as $childDir) {
                $this->_copyMainThemeDirectory($themeId, $childDir, $destination);
            }
        }

        foreach ($directory->getFiles() as $file) {
            $themeName = $themeId->getId();
            $sourcePath = $this->_getRelative($directory->getRoot());
            $destinationPath = $this->_getRelative($destination) . $this->_stringAfter($themeName, $sourcePath);

            $sourceFile = $sourcePath . DIRECTORY_SEPARATOR . $file;
            $destinationFile = $destinationPath . DIRECTORY_SEPARATOR . $file;

            try {
                $this->filesystem->copy($sourceFile, $destinationFile);
            } catch (FileNotFoundException $e) {
            }
        }
    }

    /**
     * @param $searchFor
     * @param $subject
     * @return bool|string
     */
    protected function _stringAfter($searchFor, $subject)
    {
        return substr($subject, strpos($subject, $searchFor) + strlen($searchFor));
    }

    /**
     * @param ThemeId $themeId
     * @param ThemeDirectoryInterface $directory
     * @param ThemeDirectoryRootInterface $destination
     */
    protected function _copyMainThemeStyleEditDirectory(
        ThemeId $themeId,
        ThemeDirectoryInterface $directory,
        ThemeDirectoryRootInterface $destination
    ) {
        $childDirectories = $directory->getChildren();

        if ($childDirectories) {
            foreach ($childDirectories as $childDir) {
                $this->_copyMainThemeDirectory($themeId, $childDir, $destination);
            }
        }

        foreach ($directory->getFiles() as $file) {
            // ignores files that are created by styleedit
            if (!$themeId->equals($this->sourceThemeId)
                && $this->_endsWith($directory->getRoot()->getPath(), 'styleedit')
                && $this->_endsWith($file, '.json')) {
                continue;
            }

            $themeName = $themeId->getId();
            $sourcePath = $this->_getRelative($directory->getRoot());
            $destinationPath = $this->_getRelative($destination) . $this->_stringAfter($themeName, $sourcePath);

            $sourceFile = $sourcePath . DIRECTORY_SEPARATOR . $file;
            $destinationFile = $destinationPath . DIRECTORY_SEPARATOR . $file;

            try {
                $this->filesystem->copy($sourceFile, $destinationFile);
            } catch (FileNotFoundException $e) {
            }
        }
    }

    /**
     * @param $haystack
     * @param $needle
     *
     * @return bool
     */
    protected function _endsWith($haystack, $needle)
    {
        $length = strlen($needle);
        if ($length == 0) {
            return true;
        }

        return (substr($haystack, -$length) === $needle);
    }

    /**
     * @param ThemeInterface $theme
     * @param ThemeDirectoryRootInterface $destination
     *
     * @throws FileNotFoundException
     */
    protected function _saveTheme(ThemeInterface $theme, ThemeDirectoryRootInterface $destination)
    {
        $this->_copyThemeDirectory($theme->getId(), $theme->getConfig(), $destination);
        $this->_copyThemeDirectory($theme->getId(), $theme->getHtml(), $destination);
        $this->_copyCustomHtmlDirectory($theme->getId(), $theme->getCustomHtml(), $destination);

        $this->_copyThemeDirectory($theme->getId(), $theme->getJs(), $destination);
        $this->_copyCustomJsDirectory($theme->getId(), $theme->getCustomJs(), $destination);

        $this->_copyThemeDirectory($theme->getId(), $theme->getStyles(), $destination);
        $this->_copyCustomStylesDirectory($theme->getId(), $theme->getCustomStyles(), $destination);

        $this->_copyThemeImagesDirectory($theme->getId(), $theme->getImages(), $destination);

        $this->_copyStyleEditThemeDirectory($theme->getId(), $theme->getStyleEdit(), $destination);
    }

    /**
     * @param ThemeId $themeId
     * @param                              $directory
     * @param ThemeDirectoryRootInterface $destination
     *
     * @throws FileNotFoundException
     */
    protected function _copyThemeDirectory(
        ThemeId $themeId,
        $directory,
        ThemeDirectoryRootInterface $destination
    ) {
        if ($directory instanceof ThemeDirectoryInterface) {
            $childDirectories = $directory->getChildren();
            if ($childDirectories) {
                foreach ($childDirectories as $childDirectory) {
                    $this->_copyThemeDirectory($themeId, $childDirectory, $destination);
                }
            }

            foreach ($directory->getFiles() as $file) {
                $themeName = $themeId->getId();
                $sourcePath = $this->_getRelative($directory->getRoot());
                $destinationPath = $this->_getRelative($destination) . $this->_stringAfter($themeName, $sourcePath);
                $destinationPath = $this->replaceCustomInPath(new StringType($destinationPath));

                $sourceFile = $sourcePath . DIRECTORY_SEPARATOR . $file;
                $destinationFile = $destinationPath . DIRECTORY_SEPARATOR . $file;

                try {
                    $this->filesystem->copy($sourceFile, $destinationFile);
                } catch (FileExistsException $e) {
                    $this->filesystem->delete($destinationFile);
                    $this->filesystem->copy($sourceFile, $destinationFile);
                } catch (FileNotFoundException $e) {
                }
            }
        }
    }

    /**
     * @param StringType $path
     *
     * @return string
     */
    protected function replaceCustomInPath(StringType $path)
    {
        $pat = '#\/variants\/.*custom#';
        $pathString = $path->asString();

        //  File is not inside a variants directory
        if (preg_match($pat, $pathString) === 0) {
            return $this->_stringReplaceFirst('custom', 'system', $pathString);
        }

        $pathString = $this->_stringReplaceFirst('variant/', '', $pathString);
        return $this->_stringReplaceFirst('custom', 'system', $pathString);
    }

    /**
     * @param $search
     * @param $replace
     * @param $subject
     * @return string
     */
    protected function _stringReplaceFirst($search, $replace, $subject)
    {
        $pos = strpos($subject, $search);
        if ($pos !== false) {
            return substr($subject, 0, $pos) . $replace . substr($subject, $pos + strlen($search));
        }

        return $subject;
    }

    /**
     * @param ThemeId $themeId
     * @param $directory
     * @param ThemeDirectoryRootInterface $destination
     * @throws FileNotFoundException
     */
    protected function _copyCustomHtmlDirectory(
        ThemeId $themeId,
        $directory,
        ThemeDirectoryRootInterface $destination
    ) {
        if ($directory instanceof ThemeDirectoryInterface) {
            $childDirectories = $directory->getChildren();
            if ($childDirectories) {
                foreach ($childDirectories as $childDirectory) {
                    $this->_copyCustomHtmlDirectory($themeId, $childDirectory, $destination);
                }
            }

            $overloadFilesList = [];
            $whitelist = [];

            $themeName = $themeId->getId();
            $sourcePath = $this->_getRelative($directory->getRoot());
            $destinationPath = $this->_getRelative($destination) . $this->_stringAfter($themeName, $sourcePath);
            $destinationPath = $this->replaceCustomInPath(new StringType($destinationPath));


            foreach ($directory->getFiles() as $file) {
                $sourceFile = $sourcePath . DIRECTORY_SEPARATOR . $file;
                $destinationFile = $destinationPath . DIRECTORY_SEPARATOR . $file;

                if ($this->filesystem->has($destinationFile)) {
                    $whitelist[] = $this->_extendHtmlFile($sourceFile, $destinationFile, $themeId->getId());
                } else {
                    $overloadFilesList[] = $sourceFile;
                }
            }

            if (count($overloadFilesList)) {
                //create the overloader file
                $overloaderFilename = md5(microtime()) . '.html';
                $overloaderFilePath = $destinationPath . DIRECTORY_SEPARATOR . $overloaderFilename;
                $overloaderFileContent = '';
                foreach ($overloadFilesList as $filename) {
                    $resource = $this->filesystem->readStream($filename);
                    $overloaderFileContent .= PHP_EOL . stream_get_contents($resource);
                }
                $this->filesystem->put($overloaderFilePath, $overloaderFileContent);


                //extends all the files
                $path = str_replace('public/theme/', '', $destinationPath);
                $destinationDirectory = ThemeDirectory::create($destination->withPath($path));
                foreach ($destinationDirectory->getFiles() as $fileToBeOverloaded) {
                    $filePath = $destinationPath . DIRECTORY_SEPARATOR . $fileToBeOverloaded;
                    if ($overloaderFilename !== $fileToBeOverloaded && !in_array($filePath, $whitelist)) {
                        $this->_extendHtmlFile($overloaderFilePath, $filePath,
                            array_shift(explode('.', $overloaderFilename)));
                    }
                }
            }
        }
    }

    /**
     * @param $sourceFile
     * @param $destinationFile
     * @param $suffix
     * @return string
     * @throws FileNotFoundException
     */
    protected function _extendHtmlFile($sourceFile, $destinationFile, $suffix)
    {

        $renamed = str_replace('.html', '_' . $suffix . '.html', $destinationFile);
        $newFileName = end(explode(DIRECTORY_SEPARATOR, $renamed));

        $this->filesystem->rename($destinationFile, $renamed);
        $this->filesystem->copy($sourceFile, $destinationFile);

        $extends = sprintf('{extends file="get_usermod:{$tpl_path}%s"}', $newFileName);

        $resource = $this->filesystem->readStream($destinationFile);
        $content = $extends . PHP_EOL . PHP_EOL . stream_get_contents($resource);

        $putStream = tmpfile();
        fwrite($putStream, $content);
        rewind($putStream);

        $this->filesystem->putStream($destinationFile, $putStream);

        if (is_resource($putStream)) {
            fclose($putStream);
        }
        return $renamed;
    }

    /**
     * @param ThemeId $themeId
     * @param $directory
     * @param ThemeDirectoryRootInterface $destination
     * @throws FileNotFoundException
     */
    protected function _copyCustomJsDirectory(ThemeId $themeId, $directory, ThemeDirectoryRootInterface $destination)
    {
        if ($directory instanceof ThemeDirectoryInterface) {
            $childDirectories = $directory->getChildren();
            if ($childDirectories) {
                foreach ($childDirectories as $childDirectory) {
                    $this->_copyCustomJsDirectory($themeId, $childDirectory, $destination);
                }
            }

            foreach ($directory->getFiles() as $file) {
                $themeName = $themeId->getId();
                $sourcePath = $this->_getRelative($directory->getRoot());
                $destinationPath = $this->_getRelative($destination) . $this->_stringAfter($themeName, $sourcePath);
                $destinationPath = $this->replaceCustomInPath(new StringType($destinationPath));

                $sourceFile = $sourcePath . DIRECTORY_SEPARATOR . $file;
                $destinationFile = $this->moveJsFromCustomRootDirectoryToCustomGlobalDirectory($destinationPath . DIRECTORY_SEPARATOR . $file,
                    $sourceFile);

                try {
                    $this->filesystem->copy($sourceFile, $destinationFile);
                } catch (FileExistsException $e) {
                    $this->_appendFileContent($destinationFile, $sourceFile);
                } catch (FileNotFoundException $e) {
                }
            }
        }
    }

    /**
     * Modifies the path if a file would be placed inside the root of the
     * custom directory. The file will be moved inside the global directory of
     * system directory
     *
     * This method is only called for child themes
     *
     * @param string $path
     * @param string $source
     * @return string mixed
     */
    protected function moveJsFromCustomRootDirectoryToCustomGlobalDirectory($path, $source)
    {
        $fileIsInCustomRootDirectoryPattern = '#custom/([^/]+\.js)$#';

        //  detecting if the source file is in the root of the custom directory
        if (preg_match($fileIsInCustomRootDirectoryPattern, $source) !== 0) {

            //  SHOP_ROOT ends with slash and paths starts with a slash
            $publicPath = SHOP_ROOT . substr($path, 1);

            //  Prohibiting changing of paths that already exist in the root of the system directory
            if (!file_exists($publicPath)) {

                $fileInSystemRootDirectoryPattern = '#system/([^/]+\.js)$#';

                //  the path for the directory has already been changed in a
                //  previous method without viewing the file name
                return preg_replace($fileInSystemRootDirectoryPattern, 'system/Global/$1', $path);
            }
        }

        return $path;
    }

    /**
     * @param $destinationFile
     * @param $sourceFile
     *
     * @throws FileNotFoundException
     */
    protected function _appendFileContent($destinationFile, $sourceFile)
    {
        $resource = $this->filesystem->readStream($destinationFile);

        $content = stream_get_contents($resource) . PHP_EOL . $this->filesystem->read($sourceFile);
        $putStream = tmpfile();
        fwrite($putStream, $content);
        rewind($putStream);

        $this->filesystem->putStream($destinationFile, $putStream);

        if (is_resource($putStream)) {
            fclose($putStream);
        }
    }

    /**
     * @param ThemeId $themeId
     * @param $directory
     * @param ThemeDirectoryRootInterface $destination
     * @throws FileNotFoundException
     */
    protected function _copyCustomStylesDirectory(
        ThemeId $themeId,
        $directory,
        ThemeDirectoryRootInterface $destination
    ) {
        if ($directory instanceof ThemeDirectoryInterface) {
            $childDirectories = $directory->getChildren();
            if ($childDirectories) {
                foreach ($childDirectories as $childDirectory) {
                    $this->_copyCustomStylesDirectory($themeId, $childDirectory, $destination);
                }
            }

            foreach ($directory->getFiles() as $file) {
                $themeName = $themeId->getId();
                $sourcePath = $this->_getRelative($directory->getRoot());
                $destinationPath = $this->_getRelative($destination) . $this->_stringAfter($themeName, $sourcePath);
                $destinationPath = $this->replaceCustomInPath(new StringType($destinationPath));

                $sourceFile = $sourcePath . DIRECTORY_SEPARATOR . $file;
                $destinationFile = $destinationPath . DIRECTORY_SEPARATOR . $file;

                if (!$this->filesystem->has($destinationFile)) {
                    $destinationFile = $destinationPath . DIRECTORY_SEPARATOR . 'custom' . DIRECTORY_SEPARATOR . '_usermod.scss';
                }

                try {
                    $this->filesystem->copy($sourceFile, $destinationFile);
                } catch (FileExistsException $e) {
                    $this->_appendFileContent($destinationFile, $sourceFile);
                } catch (FileNotFoundException $e) {
                }
            }
        }
    }

    /**
     * @param ThemeId $themeId
     * @param                              $imagesDirectory
     * @param ThemeDirectoryRootInterface $destination
     *
     * @throws FileNotFoundException
     */
    protected function _copyThemeImagesDirectory(
        ThemeId $themeId,
        $imagesDirectory,
        ThemeDirectoryRootInterface $destination
    ) {
        if ($imagesDirectory instanceof ThemeDirectoryInterface) {
            $childDirectories = $imagesDirectory->getChildren();
            if ($childDirectories) {
                foreach ($childDirectories as $childDirectory) {
                    $this->_copyThemeImagesDirectory($themeId, $childDirectory, $destination);
                }
            }

            foreach ($imagesDirectory->getFiles() as $file) {
                $themeName = $themeId->getId();
                $sourcePath = $this->_getRelative($imagesDirectory->getRoot());
                $destinationPath = $this->_getRelative($destination) . $this->_stringAfter($themeName, $sourcePath);

                $sourceFile = $sourcePath . DIRECTORY_SEPARATOR . $file;
                $destinationFile = $destinationPath . DIRECTORY_SEPARATOR . $file;

                try {
                    $this->filesystem->copy($sourceFile, $destinationFile);
                } catch (FileExistsException $e) {
                    $this->filesystem->delete($destinationFile);
                    $this->filesystem->copy($sourceFile, $destinationFile);
                } catch (FileNotFoundException $e) {
                }
            }
        }
    }

    /**
     * @param ThemeId $themeId
     * @param                              $directory
     * @param ThemeDirectoryRootInterface $destination
     *
     * @throws FileNotFoundException
     */
    protected function _copyStyleEditThemeDirectory(
        ThemeId $themeId,
        $directory,
        ThemeDirectoryRootInterface $destination
    ) {
        if ($directory instanceof ThemeDirectoryInterface) {
            $childDirectories = $directory->getChildren();
            if ($childDirectories) {
                foreach ($childDirectories as $childDirectory) {
                    $this->_copyThemeDirectory($themeId, $childDirectory, $destination);
                }
            }

            foreach ($directory->getFiles() as $file) {
                // ignores files that are created by styleedit
                if ($this->_endsWith($directory->getRoot()->getPath(), 'styleedit') && $this->_endsWith($file, '.json')
                    && !$themeId->equals($this->sourceThemeId)) {
                    continue;
                }

                $themeName = $themeId->getId();
                $sourcePath = $this->_getRelative($directory->getRoot());
                $destinationPath = $this->_getRelative($destination) . $this->_stringAfter($themeName, $sourcePath);
                $destinationPath = $this->replaceCustomInPath(new StringType($destinationPath));

                $sourceFile = $sourcePath . DIRECTORY_SEPARATOR . $file;
                $destinationFile = $destinationPath . DIRECTORY_SEPARATOR . $file;

                try {
                    $this->filesystem->copy($sourceFile, $destinationFile);
                } catch (FileExistsException $e) {
                    $this->filesystem->delete($destinationFile);
                    $this->filesystem->copy($sourceFile, $destinationFile);
                } catch (FileNotFoundException $e) {
                }
            }
        }
    }

    /**
     * @param Theme $theme
     * @param ThemeDirectoryRootInterface $destination
     *
     * @throws Exception
     */
    protected function _copyThemeJson(Theme $theme, ThemeDirectoryRootInterface $destination)
    {
        $jsonSource = $this->_getRelative($theme->getConfig()->getRoot()) . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'theme.json';
        $jsonDestination = $this->_getRelative($destination) . DIRECTORY_SEPARATOR . 'theme.json';

        try {
            $this->filesystem->copy($jsonSource, $jsonDestination);
        } catch (FileNotFoundException $notFoundException) {
            return;
        }
    }

    /**
     * Checks the $themeDirectoryRoot (/public/theme)
     * for files that have no matching source or .min file
     *
     * @param ThemeDirectoryRootInterface $themeDirectoryRoot
     */
    protected function findFilesThatHaveNoMatchingMinOrSourceFile(ThemeDirectoryRootInterface $themeDirectoryRoot)
    {
        $files = $this->findScriptsAndSheetsInDirectory(new StringType($themeDirectoryRoot->getPath()));

        $matchedFiles = [];

        // creating an two dimensional array matching .min files with source files
        foreach ($files as $file) {

            $notMinifiedFileName = preg_replace(self::UNMINIFIED_FILE_NAME_PATTERN, '.$2', $file);

            if (!isset($matchedFiles[$notMinifiedFileName])) {

                $matchedFiles[$notMinifiedFileName] = [];
            }

            $matchedFiles[$notMinifiedFileName][] = $file;
        }

        // detecting files that have no min or source file
        $singularFiles = array_filter($matchedFiles, static function ($item) {

            return count($item) === 1 ? array_shift($item) : null;
        });

        //  creating a one dimensional none associative array
        $singularFiles = array_values(array_map('array_shift', $singularFiles));

        $this->duplicateFilesThatHaveNoMatchingMinOrSourceFile($singularFiles);
    }

    /**
     * @param StringType $directory
     * @return string[]
     */
    protected function findScriptsAndSheetsInDirectory(StringType $directory)
    {
        $result = [];

        foreach (new DirectoryIterator($directory->asString()) as $file) {

            if (!$file->isDot()) {

                $fileName = new StringType($file->getPath() . DIRECTORY_SEPARATOR . $file->getFilename());

                if ($file->isDir()) {

                    foreach ($this->findScriptsAndSheetsInDirectory($fileName) as $recursivelyFoundFile) {

                        $result[] = $recursivelyFoundFile;
                    }

                } elseif ($file->isFile() && preg_match(self::FILE_IS_SCRIPT_OR_SHEET_PATTERN, $fileName->asString())) {

                    $result[] = $fileName->asString();
                }
            }
        }

        return $result;
    }

    /**
     * Creates a duplicate of a file with .min if only the source file exists
     * or a file without .min if only the .min file exists
     *
     * @param string[] $files
     */
    protected function duplicateFilesThatHaveNoMatchingMinOrSourceFile(array $files)
    {
        foreach ($files as $file) {

            if (preg_match(self::UNMINIFIED_FILE_NAME_PATTERN, $file, $result)) {

                //  file only exists as source file
                if ($result[1] === '') {

                    $destination = preg_replace(self::UNMINIFIED_FILE_NAME_PATTERN, '.min.$2', $file);
                } //  File only exists as .min file
                else {

                    $destination = preg_replace(self::UNMINIFIED_FILE_NAME_PATTERN, '.$2', $file);
                }

                copy($file, $destination);
            }
        }
    }
}
